$ontext

First-written:       <2010/06/10>
Time-stamp:          <2019-08-21 17:45:03 st>

Takeda, Shiro, Arimura, Toshi H. and Sugino, Makoto (2019) "Labor Market
Distortions and Welfare-Decreasing International Emissions Trading."
Environmental and Resource Economics, https://doi.org/10.1007/s10640-018-00317-4

$offtext
$title A model for analyzing international emissions trading.
display "@ A model for analyzing international emissions trading.";

*       End-of-line comment symbol:
$eolcom //

* option solprint=on;

*       ----------------------------------------------------------------------
*       Definition of control variables:

*       Scenario name:
$if not setglobal s_name $setglobal s_name scn_default

*       Dataset:
$if not setglobal ds $setglobal ds .\data\gtp15_8_0001

*       Patterns of permit trade
*       No trade in all regions
$if not setglobal ntr $setglobal ntr 1

*       Perfect trade
$if not setglobal tr $setglobal tr 1

*       International capital movement
$if not setglobal cap_reg $setglobal cap_reg 1

*       The file name for specifying CO2 reduction rates:
$if not setglobal rdr_file $setglobal rdr_file .\scenarios\rdr_default.gms
* $setglobal rdr_file rdr_file_usa.gms

*       The file name for specifying policy choice:
$if not setglobal pol_file $setglobal pol_file .\scenarios\pol_default.gms
* $setglobal rdr_file rdr_file_usa.gms

*       Settings for permit trade:
$if not setglobal ptrade_file $setglobal ptrade_file .\scenarios\ptrade_default.gms

*       Run only Benchmark replication.
$if not setglobal br_only $setglobal br_only 0

*       Run only cleanup calculation:
$if not setglobal cc_only $setglobal cc_only 0

*       Run scale shock experiment:
$if not setglobal scale_shock $setglobal scale_shock 0

*       Scale factor for EOS:
$if not setglobal s_eos $setglobal s_eos 1

*       Supply elasticity of fossil fuel:
$if not setglobal eos_ff $setglobal eos_ff 2

*       Leisure (variable labor supply):
$if not setglobal fl_lei $setglobal fl_lei 1

*       Flag for minimum wage model
$if not setglobal fl_mw $setglobal fl_mw 0

*       Wage curve model:
$if not setglobal fl_wc $setglobal fl_wc 0

*       Elasticity of unemployment to real wage rate:
$if not setglobal wc_elas $setglobal wc_elas -0.1

*       Benchmark unemployment rate used for wage curve model:
$if not setglobal wc_urate_file $setglobal wc_urate_file .\scenarios\wc_urate_file_default.gms

*       Scale for unemployment rate
$if not setglobal wc_urate_sc $setglobal wc_urate_sc 1

*       Labor supply elasticity (compensated):
$if not setglobal eps_l $setglobal eps_l 0.1

*       Labor supply elasticity (uncompensated):
$if not setglobal eps_l_u $setglobal eps_l_u 0.3

*       Labor tax rate for developed countries:
$if not setglobal ltax_dve $setglobal ltax_dve 0.4

*       Labor tax rate for less developed countries:
$if not setglobal ltax_ldv $setglobal ltax_ldv 0.2

*       Scale for reduction rate
$if not setglobal s_rd $setglobal s_rd 1

$if not setglobal gdx_f $setglobal gdx_f .\results\%s_name%_results.gdx

*       File name for displaying title in DOS window.
file titlemaker /'titlemaker_%s_name%.cmd'/;
putclose titlemaker '@title Solving scenario = %s_name%' /;
execute 'titlemaker_%s_name%.cmd';

$set ghgdata yes

set     allr    /
        JPN
        USA
        EUR
        OOE
        RUS
        CHN
        IND
        ROW
        subj
        nsubj
        /
        alli    /
        AGR, COL, CRU, GAS, OMN, PPP, OIL, CRP, NMM, I_S, NFM, TRN, OMF, ELE, SER,
        CGD     Savings good,
        HH      Household,
        SUM     Summation
        /;
display allr, alli;

set     pall            All patterns of permit trade    / BAU, NTR, TR /
        psol(pall)      Patterns solved /
                BAU             Benchmark equilibrium
$if not %ntr%==0       NTR     No trade in all regions
$if not %tr%==0        TR      International permit trade
/
        psol_(pall)
        pcur(pall)
        pbau(pall)
        pss(pall)
;
psol_(pall) = psol(pall);
psol_("bau") = no;
pbau("bau") = yes;

*       ----------------------------------------------------------------------
*       Benchmark data:
display "@ Benchmark data:";

* $offlisting
$include gtap8data.gms
* $onlisting

alias (i,ii), (g,gg), (r,rr), (f,ff);
display r, f, i, g;

display vfm, vdfm, vifm, vxmd, vst, vtwr, evd, evt, eco2d, eco2i, rto, rtf, rtfd, rtfi,
        rtxs, rtms, esubd, esubva, esubm, etrae, eta, epsilon;

set     r_s(r,s)
        r_r(r,s);
r_s(r,s) = yes;
r_s(r,r) = no;

r_r(r,s) = no;
r_r(r,r) = yes;

parameters
    vdgm(i,r)   Government - domestic purchases at market prices,
    vigm(i,r)   Government - imports at market prices,
    vdpm(i,r)   Private households - domestic purchases at market prices,
    vipm(i,r)   Private households - imports at market prices
    vdim(i,r)   Investment demand (domestic)
    viim(i,r)   Investment demand (import)
;
vdgm(i,r) = vdfm(i,"g",r);
vigm(i,r) = vifm(i,"g",r);
vdpm(i,r) = vdfm(i,"c",r);
vipm(i,r) = vifm(i,"c",r);
vdim(i,r) = vdfm(i,"i",r);
viim(i,r) = vifm(i,"i",r);
display vdgm, vigm, vdpm, vipm, vdim, viim;

parameter
    rtpd(i,r)   Private domestic consumption taxes
    rtpi(i,r)   Private import consumption tax rates
    rtgd(i,r)   Government domestic rates
    rtgi(i,r)   Government import tax rates
    rtid(i,r)   Investment domestic rates
    rtii(i,r)   Investment import tax rates
;
rtpd(i,r) = rtfd(i,"c",r);
rtpi(i,r) = rtfi(i,"c",r);
rtgd(i,r) = rtfd(i,"g",r);
rtgi(i,r) = rtfi(i,"g",r);
rtid(i,r) = rtfd(i,"i",r);
rtii(i,r) = rtfi(i,"i",r);

parameter
        rto0(g,r)       Output (or income) subsidy rates
        rtpd0(i,r)      Private domestic consumption taxes
        rtpi0(i,r)      Private import consumption tax rates
        rtgd0(i,r)      Government domestic rates
        rtgi0(i,r)      Government import tax rates
        rtid0(i,r)      Investment domestic rates
        rtii0(i,r)      Investment import tax rates
;
rto0(g,r) = rto(g,r);
rtpd0(i,r) = rtpd(i,r);
rtpi0(i,r) = rtpi(i,r);
rtgd0(i,r) = rtgd(i,r);
rtgi0(i,r) = rtgi(i,r);
rtid0(i,r) = rtid(i,r);
rtii0(i,r) = rtii(i,r);

display rto0, rtf0, rtfd0, rtfi0, rtpd0, rtpi0, rtgd0, rtgi0, rtid0, rtii0, rtfd0, rtfi0, rtxs0, rtms0;

parameter
    vinv(r)     Investment;
vpm(r) = sum(i, (1+rtpd(i,r))*vdpm(i,r) + (1+rtpi(i,r))*vipm(i,r));
vgm(r) = sum(i, (1+rtgd(i,r))*vdgm(i,r) + (1+rtgi(i,r))*vigm(i,r));
vinv(r) = sum(i, (1+rtid(i,r))*vdim(i,r) + (1+rtii(i,r))*viim(i,r));
* vinv(r) = vom("i",r);
display vinv;

*       ----------------------------------------------------------------------
*       Remove small values:
display "@ Remove small values:";

parameter chk_vifm, chk_vdfm, chk_vipm, chk_vdpm, chk_vigm, chk_vdgm, chk_vim, chk_vxmd;

chk_vifm(i,g,r)$(vifm(i,g,r) < 1e-6) = 1;
chk_vdfm(i,g,r)$(vdfm(i,g,r) < 1e-6) = 1;
chk_vipm(i,r)$(vipm(i,r) < 1e-6) = 1;
chk_vdpm(i,r)$(vdpm(i,r) < 1e-6) = 1;
chk_vigm(i,r)$(vigm(i,r) < 1e-6) = 1;
chk_vdgm(i,r)$(vdgm(i,r) < 1e-6) = 1;
chk_vim(i,r)$(vim(i,r) < 1e-6) = 1;
chk_vxmd(i,r,s)$(vxmd(i,r,s) < 1e-6) = 1;
display chk_vifm, chk_vdfm, chk_vipm, chk_vdpm, chk_vigm, chk_vdgm, chk_vim, chk_vxmd;

vifm(i,j,r) = round(vifm(i,j,r), 6);
vdfm(i,j,r) = round(vdfm(i,j,r), 6);
vipm(i,r) = round(vipm(i,r), 6);
vdpm(i,r) = round(vdpm(i,r), 6);
vigm(i,r) = round(vigm(i,r), 6);
vdgm(i,r) = round(vdgm(i,r), 6);
vim(i,r) = round(vim(i,r), 6);
vxmd(i,r,s) = round(vxmd(i,r,s), 6);

*       ----------------------------------------------------------------------
*       Parameters for international emission trading:
$ontext
ptr_p: non-zero if IET is possible between some regions
ptr(r): non-zero if region r participates in IET.
pntr(r): non-zero if region r does not participate in IET
$offtext
parameter
    ptr_p       Non-zero if international permit trade is possible for any region
    ptr(r)      Non-zero if international permit trade is possible for region r
    pntr(r)     Non-zero if international permit trade is not possible for region r
;
ptr_p = 0;
ptr(r) = 0;
pntr(r) = 1;
display ptr, pntr;

*       ----------------------------------------------------------------------
*       Switch for region-specific or global:
scalar
    rsk Flag for region-specific capital
    gk  Flag for global capital
;
if(%cap_reg%,
    rsk = 1;
    gk = 0;
else
    rsk = 0;
    gk = 1;
);
display rsk, gk;

display r;

set
    jpn(r)      Japan   / jpn /
    usa(r)      USA     / usa /
    eur(r)      EU27    / eur /
    chn(r)      China   / chn /
;

*       ----------------------------------------------------------------------
*       Energy Sectors and carbon emissions:
display "@ Energy Sectors and carbon emissions:";

set
    ele(i)      Electricity                             / ele /
    oil(i)      Petroleum and coal products (refined)   / oil /
    gas(i)      Gas                                     / gas /
    en(i)       Emissions sources                       / oil, gas, col, cru /
    xe(i)       Exhaustible energy                      / cru, gas, col /
    fe(i)       Final energy                            / oil, gas, col /
    lqd(i)      GAS and OIL                             / oil, gas /
    e(i)        Energy markets
                / gas     Natural gas works
                  ele     Electricity and heat
                  oil     Petroleum and coal products (refined)
                  col     Coal transformation
                  cru     Crude oil /
    nr(i,r)     Non-exhaustible energy production
    crp(i)      Chemical industry                       / crp /
    ncrp(i)     Non-chemical industry
    crpc(i)     OIL and CRP                             / oil, crp /
    oth(i,r)    Other sectors
    k(f)        Global capital                          / cap /
    ssec(*)     / hh, sum /
;

nr(i,r) = yes$(vom(i,r)$(not xe(i)));
ncrp(i) = yes$(not crp(i));
oth(i,r) = yes$(vom(i,r) and (not oil(i)) and (not xe(i)) and (not crp(i)));
display nr, ncrp, oth;

set
    dve(r)      Developed regions / usa, jpn, eur, ooe /
;
display dve;

*       ----------------------------------------------------------------------
*       Parameters for labor market:
display "@ Parameters for labor market:";

parameter
    fl_lei      Model with leisure (endogenous labor supply)
;
fl_lei = 0;
fl_lei$%fl_lei% = 1;
display fl_lei;

parameter
    fl_mw       Minimum wage model
    fl_wc       Wage curve model
    wc_elas     Wage curve elasticity
    v0          Benchmark real wage rate
    ur0         Benchmark unemployment rate
    unempr0     Benchmark unemployment
;

fl_mw = %fl_mw%;
fl_wc = %fl_wc%;
wc_elas = %wc_elas%;
v0(r) = 1;

display fl_mw;
abort$(fl_mw and fl_lei) "fl_lei must be zero if fl_mw is non-zero!";

display fl_mw, fl_wc, wc_elas, v0;

* gams-include-file: .\scenarios\wc_urate_file_default.gms
$include %wc_urate_file%
ur0(r) = ur0(r) * %wc_urate_sc%;
display ur0;
;
*       ----------------------------------------------------------------------
*       parameters used for model specification:
display "@ parameters used for model specification:";

parameter
    vc(r)       Final consumption demand
;
vc(r) = vpm(r);
display vc;

*       ----------------------------------------------------------------------
*       Parameters for fossil-fuel sectors:
display "@ Parameters for fossil-fuel sectors:";

parameters
    vrd(i,r)        Resource inputs for fossil fuel production
    esub_es(i,r)    Elasticity of substitution in energy production
    thetar(i,r)     Value share of resource inputs in fossil supply
    etaf(xe)        Elasticity of supply
;

*       Price elasticities of fossil fuel supply:
etaf("cru")  = %eos_ff%;
etaf("gas")  = %eos_ff%;
etaf("col")  = %eos_ff%;

*       ----------------------------------------------------------------------
*       Establish resource inputs for resource supply:
vrd(xe,r) = vfm("nrs", xe,r);

*       Share of resource input:
thetar(xe,r)$vom(xe,r) = vrd(xe,r)/(vom(xe,r)*(1-rto(xe,r)));

*       Elasticity of substitution between resource vs non-resource:
esub_es(xe,r)$(1-thetar(xe,r)) = thetar(xe,r)*etaf(xe)/(1-thetar(xe,r));

display thetar, esub_es;

*       ----------------------------------------------------------------------
*       Additional parameters used for model specification:
display "@ Additional parameters used for model specification:";

parameters
    vafm(i,j,r)         Aggregate intermediate inputs
    vapm(i,r)           Private expenditures
    vagm(i,r)           Governent expenditures
    vaim(i,r)           Investment expenditures
    vafma(j,r)          Armington intermediate input
;

vafm(i,j,r) = vdfm(i,j,r)*(1+rtfd0(i,j,r)) + vifm(i,j,r)*(1+rtfi0(i,j,r));
vapm(i,r)   = vdpm(i,r)*(1+rtpd0(i,r)) + vipm(i,r)*(1+rtpi0(i,r));
vagm(i,r)   = vdgm(i,r)*(1+rtgd0(i,r)) + vigm(i,r)*(1+rtgi0(i,r));
vaim(i,r)   = vdim(i,r)*(1+rtid0(i,r)) + viim(i,r)*(1+rtii0(i,r));

* display vafm, vapm;

vafma(j,r) = sum(fe, vafm(fe,j,r));
* display vafma;

parameter
    chk_vafm_oil
    chk_vafm_crp
    chk_vafm_cru_use;

chk_vafm_oil(i,r) = vafm(i,"oil",r);
chk_vafm_crp(i,r) = vafm(i,"crp",r);

chk_vafm_cru_use(j,r) = round(vafm("cru",j,r), 4);

* display chk_vafm_oil, chk_vafm_crp, chk_vafm_cru_use;

$ontext Divde intermediate energy input in sector CRP (chemical sector) to 1)
input for combustion use and input for non-combustion use.

$offtext
table fs_rate   Feedstock rate
                USA         JPN         OOE         EUR         RUS         CHN         IND         ROW
OIL.CRP       0.915       0.734       1.835      14.458       0.294       0.561       0.715      27.734
GAS.CRP       0.251       0.297       2.234      10.007       0.893       0.582       0.131      17.695
;
display fs_rate;

parameters
  vafm_nc(i,crp,r)   value of intermediate inputs used for non-combustion use
  vafm_c(i,crp,r)    value of intermediate inputs used for combustion use
  vdfm_nc(i,crp,r)   dom intermediate inputs used for non-combustion use
  vdfm_c(i,crp,r)    dom intermediate inputs used for combustion use
  vifm_nc(i,crp,r)   imp intermediate inputs used for non-combustion use
  vifm_c(i,crp,r)    imp intermediate inputs used for combustion use
;

vdfm_nc(i,crp,r) = 0.01 * fs_rate(i,crp,r) * vdfm(i,crp,r);
vdfm_c(i,crp,r) = vdfm(i,crp,r) - vdfm_nc(i,crp,r);

vifm_nc(i,crp,r) = 0.01 * fs_rate(i,crp,r) * vifm(i,crp,r);
vifm_c(i,crp,r) = vifm(i,crp,r) - vifm_nc(i,crp,r);

* display vdfm_nc, vifm_nc, vdfm_c, vifm_c;

vafm_nc(i,crp,r)
    = (1 + rtfd0(i,crp,r)) * vdfm_nc(i,crp,r)
    + (1 + rtfi0(i,crp,r)) * vifm_nc(i,crp,r);

vafm_c(i,crp,r)
    = (1 + rtfd0(i,crp,r)) * vdfm_c(i,crp,r)
    + (1 + rtfi0(i,crp,r)) * vifm_c(i,crp,r);

display vafm_nc, vafm_c;

*       ----------------------------------------------------------------------
*       Generate base year carbon dioxide emissions by sector:
display "@ Generate base year carbon dioxide emissions by sector:";

parameter
    co2fd(en,j,r)       "Emissions from domestic product in current production (mtoe CO2)"
    co2fi(en,j,r)       "Emissions from imports in current production (mtoe CO2)"
    co2clp(r)           "Emissions from CLP sector (mtoe CO2)"
    co2hd(en,r)         "Emissions from input i in households (mtoe CO2)"
    co2hi(en,r)         "Emissions from input i in households (mtoe CO2)"
    co2gd(en,r)         "Emissions from input i in government (mtoe CO2)"
    co2gi(en,r)         "Emissions from input i in government (mtoe CO2)"

    co2bench(*)         "Benchmark CO2 emission in region r (MtCO2)"
    co2bench_chk(*)     "Benchmark CO2 emission in region r (MtCO2)"
    co2lim(r)           "CO2 emission limit (MtCO2)"
;
parameter       co2iea(*)       "CO2 emissions from fuel combustion (2007, IEA)" /
                USA     5763
                JPN     1233
                OOE     990
                EUR     3757
                RUS     1566
                CHN     6316
                IND     1357
                ROW     8286
*               World   29268
                /
;
co2iea("World") = sum(r, co2iea(r));
display co2iea;

co2fd(en,j,r) = eco2d(en,j,r);
co2fi(en,j,r) = eco2i(en,j,r);
co2hd(en,r) = eco2d(en,"c",r);
co2hi(en,r) = eco2i(en,"c",r);
co2gd(en,r) = eco2d(en,"g",r);
co2gi(en,r) = eco2i(en,"g",r);

display "Before adjustment", co2fd, co2fi, co2hd, co2hi, co2gd, co2gi;

$ontext
If vafm and vapm are zero, set zero to CO2 from these sectors.
$offtext
co2fd(en,j,r)$(not vdfm(en,j,r)) = 0;
co2fi(en,j,r)$(not vifm(en,j,r)) = 0;
co2fd(en,crp,r)$(not vdfm_c(en,crp,r)) = 0;
co2fi(en,crp,r)$(not vifm_c(en,crp,r)) = 0;
co2hd(en,r)$(not vdpm(en,r)) = 0;
co2hi(en,r)$(not vipm(en,r)) = 0;
co2gd(en,r)$(not vdgm(en,r)) = 0;
co2gi(en,r)$(not vigm(en,r)) = 0;

co2bench_chk(r) = sum((en,j), co2fd(en,j,r) + co2fi(en,j,r))
    + sum(en, co2hd(en,r) + co2hi(en,r))
    + sum(en, co2gd(en,r) + co2gi(en,r));
co2bench_chk("World") = sum(r, co2bench_chk(r));
display co2bench_chk;

$ontext
+ Modify CO2 data by IEA CO2 dataf.
+ Modify CO2 emissions from Iron and Steel in Japan.
$offtext
parameter
    adj_co2;
adj_co2(r) = co2iea(r) / co2bench_chk(r);
display adj_co2;

co2fd(en,j,r) = co2fd(en,j,r) * adj_co2(r);
co2fi(en,j,r) = eco2i(en,j,r) * adj_co2(r);
co2hd(en,r) = eco2d(en,"c",r) * adj_co2(r);
co2hi(en,r) = eco2i(en,"c",r) * adj_co2(r);
co2gd(en,r) = eco2d(en,"g",r) * adj_co2(r);
co2gi(en,r) = eco2i(en,"g",r) * adj_co2(r);

co2bench_chk(r) = sum((en,j), co2fd(en,j,r) + co2fi(en,j,r))
    + sum(en, co2hd(en,r) + co2hi(en,r))
    + sum(en, co2gd(en,r) + co2gi(en,r));
co2bench_chk("World") = sum(r, co2bench_chk(r));
display co2bench_chk;

parameters
    chk_co2f_i_s_jpn
    chk_co2f_i_s_jpn_sh;

chk_co2f_i_s_jpn(en,"d") = co2fd(en,"i_s","jpn");
chk_co2f_i_s_jpn(en,"i") = co2fi(en,"i_s","jpn");
chk_co2f_i_s_jpn(en,"sum") = chk_co2f_i_s_jpn(en,"d") + chk_co2f_i_s_jpn(en,"i");
chk_co2f_i_s_jpn("sum","sum") = sum(en, chk_co2f_i_s_jpn(en,"sum"));
display chk_co2f_i_s_jpn;

chk_co2f_i_s_jpn_sh(en,"d") = chk_co2f_i_s_jpn(en,"d") / chk_co2f_i_s_jpn(en,"sum");
chk_co2f_i_s_jpn_sh(en,"i") = chk_co2f_i_s_jpn(en,"i") / chk_co2f_i_s_jpn(en,"sum");
chk_co2f_i_s_jpn_sh(en,"sum") = chk_co2f_i_s_jpn_sh(en,"d") + chk_co2f_i_s_jpn_sh(en,"i");
display chk_co2f_i_s_jpn_sh;

co2fd("oil","i_s","jpn") = co2fd("oil","i_s","jpn") + 80 * chk_co2f_i_s_jpn_sh("oil","d");
co2fi("oil","i_s","jpn") = co2fi("oil","i_s","jpn") + 80 * chk_co2f_i_s_jpn_sh("oil","i");

co2fd("col","i_s","jpn") = co2fd("col","i_s","jpn") + 20 * chk_co2f_i_s_jpn_sh("col","d");
co2fi("col","i_s","jpn") = co2fi("col","i_s","jpn") + 20 * chk_co2f_i_s_jpn_sh("col","i");

chk_co2f_i_s_jpn(en,"d") = co2fd(en,"i_s","jpn");
chk_co2f_i_s_jpn(en,"i") = co2fi(en,"i_s","jpn");
chk_co2f_i_s_jpn(en,"sum") = chk_co2f_i_s_jpn(en,"d") + chk_co2f_i_s_jpn(en,"i");
chk_co2f_i_s_jpn("sum","sum") = sum(en, chk_co2f_i_s_jpn(en,"sum"));
display chk_co2f_i_s_jpn;

parameter
    chk_co2f_oil
    chk_co2f_crp;
chk_co2f_oil(en,r) = co2fd(en,"oil",r) + co2fi(en,"oil",r);
chk_co2f_crp(en,r) = co2fd(en,"crp",r) + co2fi(en,"crp",r);
display chk_co2f_oil, chk_co2f_crp;

co2bench(r) = sum((en,j), co2fd(en,j,r) + co2fi(en,j,r))
    + sum(en, co2hd(en,r) + co2hi(en,r))
    + sum(en, co2gd(en,r) + co2gi(en,r));
co2bench("World") = sum(r, co2bench(r));
display co2bench;


co2lim(r) = co2bench(r) * 10;
display co2lim;

$ontext
Check CO2 data.
$offtext
parameter
    chk_co2_use
    chk_co2_use_oil
    chk_co2_so
    chk_co2_so_sh
    chk_co2int          "CO2 intensity (tCO2/mil$)"
    chk_eneint          Energy intensity
;

chk_co2_use(j,r) = sum(en, co2fd(en,j,r) + co2fi(en,j,r));
chk_co2_use("c",r) = sum(en, co2hd(en,r) + co2hi(en,r));
chk_co2_use("g",r) = sum(en, co2gd(en,r) + co2gi(en,r));
chk_co2_use("sum",r) = sum(j, chk_co2_use(j,r)) + chk_co2_use("c",r) + chk_co2_use("g",r);
display chk_co2_use;

chk_co2_use_oil(en,r) = co2fd(en,"oil",r) + co2fi(en,"oil",r);
display chk_co2_use_oil;

chk_co2_so(en,r) = sum(j, co2fd(en,j,r)) + co2hd(en,r) + co2gd(en,r)
    + sum(j, co2fi(en,j,r)) + co2hi(en,r) + co2gi(en,r);
chk_co2_so("sum",r) = sum(en, chk_co2_so(en,r));

chk_co2_so_sh(en,r) = 100 * chk_co2_so(en,r) / chk_co2_so("sum",r);
display chk_co2_so, chk_co2_so_sh;

chk_co2int(i,r)$vom(i,r) = 1000 * sum(en, co2fd(en,i,r) + co2fi(en,i,r)) / vom(i,r);
chk_co2int("c",r) = 1000 * sum(en, co2hd(en,r) + co2hi(en,r)) / vc(r);
* chk_co2int("g",r) = 1000 * sum(en, co2gd(en,r) + co2gi(en,r)) / vg(r);
display chk_co2int;

chk_eneint(i,r)$vom(i,r) =
    100 * sum(j$e(j), vafm(j,i,r)) / ((1-rto(i,r)) * vom(i,r));
display chk_eneint;

*       ----------------------------------------------------------------------
*       Divide intermediate inputs for non-cumbustion use and combustion use:
display "@ Divide intermediate inputs for non-cumbustion use and combustion use:";
$ontext

(1) From vdfm(i,crp,r), vifm(i,crp,r), vdfm_nc(i,crp,r), vifm_nc(i,crp,r), we have

vdfm_c(i, crp,r)   = vdfm(i, crp, r) - vdfm_nc(i, crp, r);
vifm_c(i, crp, r)  = vifm(i, crp, r) - vifm_nc(i, crp, r);

(2) From vdfm_c, vifm_c, vdfm_nc, vifm_nc, we have

vafm_nc(i,crp,r) =
    (1 + rtfd0(i,crp,r)) * vdfm_nc(i,crp,r)
    + (1 + rtfi0(i,crp,r)) * vifm_nc(i,crp,r);

vafm_c(i,crp,r)
    = (1 + rtfd0(i,crp,r)) * vdfm_c(i,crp,r)
    + (1 + rtfi0(i,crp,r)) * vifm_c(i,crp,r)
    ;
$offtext
parameters
  vafm_c(i,crp,r)    value of intermediate inputs used for combustion use
  vdfm_c(i,crp,r)    dom intermediate inputs used for combustion use
  vifm_c(i,crp,r)    imp intermediate inputs used for combustion use
;

vdfm_c(i,crp,r)  = vdfm(i,crp,r) - vdfm_nc(i,crp,r);
vifm_c(i,crp,r)  = vifm(i,crp,r) - vifm_nc(i,crp,r);

display vdfm_nc, vifm_nc, vdfm_c, vifm_c;

vafm_nc(i,crp,r)
    = (1 + rtfd0(i,crp,r)) * vdfm_nc(i,crp,r)
    + (1 + rtfi0(i,crp,r)) * vifm_nc(i,crp,r);

vafm_c(i,crp,r)
    = (1 + rtfd0(i,crp,r)) * vdfm_c(i,crp,r)
    + (1 + rtfi0(i,crp,r)) * vifm_c(i,crp,r)
    ;
display vafm_nc, vafm_c;

*       ----------------------------------------------------------------------
*       Labor supply and leisure:
display "@ Labor supply and leisure:";

parameters
    esub_lsr(r) EOS between leisure and consumption

    eps_l      Compensated labor supply elasticity
    eps_l_u    Uncompensated labor supply elasticity
    theta_l    Leisure's share in overall budget

    vlsr(r)    Benchmark value of leisure
    vls(r)     Benchmark labor supply
    vld(i,r)   Base year labor demand by sector
    evoa(r)    Value of labour income
    l_bar(r)   Benchmark labor endowment
    ltax0(r)   Benchmark tax on labor (gross base)

    ballard(r) Ballard coefficient
    plh0(r)    Reference wage rate for household

    vu(r)      Benchmark utility
    vtax(r)    Implicit lump-sum tax
;

*       ----------------------------------------------------------------------
*       Labor, leisure for regions other than Japan:

*       Benchmark tax on labor (gross base):
ltax0(r)$dve(r) = %ltax_dve%;
ltax0(r)$(not dve(r)) = %ltax_ldv%;
display ltax0;

*       Base year labor demand by sector:
vld(i,r) = vfm("lab",i,r)*(1+rtf0("lab",i,r));

*       Benchmark labor supply:
vls(r) = sum(i, vld(i,r));

*       Value of labour income:
evoa(r) = (1 - ltax0(r)) * vls(r);

*       Labor supply elasticity:
*eps_l =   0.2;
*eps_l_u = 0.29;
eps_l   = %eps_l%;
eps_l_u = %eps_l_u%;

*       Leisure's share in overall budget:
theta_l = eps_l_u - eps_l;

*       Benchmark value of leisure:
vlsr(r) = vc(r) * theta_l / (1-theta_l);

*       Benchmark labor endowment:
l_bar(r) = vls(r) + vlsr(r)$fl_lei;

*       Elasticity of subst. between leisure and consumption:
esub_lsr(r) = (eps_l_u/(1- theta_l)) * vls(r) / vlsr(r);

*       Ballard coefficient
ballard(r) = (l_bar(r) / vls(r));

display esub_lsr, vls, vlsr, ballard;

*       Adjustment labor supply:
l_bar(r)$(fl_wc or fl_mw) = l_bar(r) / (1 - ur0(r));

*       Benchmark unemployment:
unempr0(r) = ur0(r) * l_bar(r);
display l_bar, unempr0;

*       Reference price of labor for household
plh0(r) = (1 - ltax0(r));

*       Benchmark utility:
vu(r) = vc(r) + plh0(r)*vlsr(r)$fl_lei;
display vu;

*       Calculation of government transfer
vtax(r) =
    evom("cap",r) + plh0(r)*l_bar(r) - plh0(r)*unempr0(r)$(fl_wc or fl_mw)
    + evom("lnd", r) + evom ("nrs", r) + vb(r)
    - (vu(r) + vinv(r));
display vtax;

*       ----------------------------------------------------------------------
*       Elasticity of substitution
parameter
    esubc       EOS between energy and non-energy consumption
    esubec      EOS for energy consumption
    esubcc      EOS for non-energy consumption
    esubvae(i)  EOS for VA and energy
    esube(i)    EOS for electricity and non-electricity
    esubnel(i)  EOS for coal and non-coal
    esublqd(i)  EOS for liquidity energy
;

esubc = 0.5 * %s_eos%;
esubec = 1 * %s_eos%;
esubcc = 1 * %s_eos%;

esubvae(i) = 0.5 * %s_eos%;
esube(i) = 0.1 * %s_eos%;
esubnel(i) = 0.5 * %s_eos%;
esublqd(i) = 2 * %s_eos%;

esubva(i) = esubva(i) * %s_eos%;

*       Armington elasticity
esubd(i) = esubd(i) * %s_eos%;
esubm(i) = esubm(i) * %s_eos%;

etrae("nrs") = 0;
etrae(f) = etrae(f) * %s_eos%;
display etrae;

vb(r) =
    vpm(r) + vgm(r) + vinv(r)
    - sum(f, evom(f,r))
    - sum(j, vom(j,r)*rto0(j,r))
    - sum(j, sum(f, vfm(f,j,r)*rtf0(f,j,r)))
    - sum(j, sum(i, vdfm(i,j,r)*rtfd0(i,j,r) + vifm(i,j,r)*rtfi0(i,j,r)))
    - sum(i, vdpm(i,r)*rtpd0(i,r) + vipm(i,r)*rtpi0(i,r))
    - sum(i, vdgm(i,r)*rtgd0(i,r) + vigm(i,r)*rtgi0(i,r))
    - sum(i, vdim(i,r)*rtid0(i,r) + viim(i,r)*rtii0(i,r))
    - sum((i,s), rtms0(i,s,r) *  (vxmd(i,s,r) * (1-rtxs0(i,s,r)) + sum(j,vtwr(j,i,s,r))))
    + sum((i,s), rtxs0(i,r,s) * vxmd(i,r,s));
vb("chksum") = sum(r, vb(r));
display vb;

parameter
    chk_vlabt
    chk_vlumpt
    chk_votht
    chk_vtax
    chk_gov
;
chk_vlabt("v",r) = ltax0(r) * vls(r);
chk_vlumpt("v",r) = vtax(r);
chk_votht("v",r) =
*       Output tax
    sum(j, rto0(j,r) * vom(j,r))
*       Sluggish factor tax:
    + sum(j, sum(sf, rtf0(sf,j,r) * vfm(sf,j,r)))
*       Capital tax:
    + sum(j, rtf0("cap",j,r) * vfm("cap",j,r))
*       Intermediate input tax:
    + sum((i,j)$vdfm(i,j,r), rtfd0(i,j,r) * vdfm(i,j,r))
    + sum((i,j)$vifm(i,j,r), rtfi0(i,j,r) * vifm(i,j,r))
*       Tax on consumption demand:
    + sum(i$vdpm(i,r), rtpd0(i,r) * vdpm(i,r))
    + sum(i$vipm(i,r), rtpi0(i,r) * vipm(i,r))
*       Tax on government demand:
    + sum(i$vdgm(i,r), rtgd0(i,r) * vdgm(i,r))
    + sum(i$vigm(i,r), rtgi0(i,r) * vigm(i,r))
*       Tax on investment demand:
    + sum(i$vdim(i,r), rtid0(i,r) * vdim(i,r))
    + sum(i$viim(i,r), rtii0(i,r) * viim(i,r))
*       Export subsidy
    - sum((i,s)$vxmd(i,r,s), rtxs0(i,r,s) * vxmd(i,r,s))
*       Tariff
    + sum((i,s)$vxmd(i,s,r),
        rtms0(i,s,r) * (1 - rtxs0(i,s,r)) * vxmd(i,s,r))
    + sum((i,s,j)$vtwr(j,i,s,r), rtms0(i,s,r) * vtwr(j,i,s,r))
    ;
chk_vtax("v",r) = chk_vlabt("v",r) + chk_vlumpt("v",r) + chk_votht("v",r);

chk_gov("v",r) = chk_vtax("v",r) - vgm(r);

display chk_vlabt, chk_vlumpt, chk_votht, chk_vtax, chk_gov;

*       ----------------------------------------------------------------------
*       Sets and parameters for policy.
set
    cap(i,r)    Sectors subject to emission cap
    subj(r)     Countries subject to emission cap
;
parameter
    rd_rate(r)  Reduction rate in terms of the base year emissions (%)
;

*       Initialization:
subj(r) = no;
rd_rate(r) = 0;
cap(i,r) = yes;
display "The default value", subj, rd_rate, cap;

parameter
    fl_lump(r)          "Yes if lump(r) is valid"
    fl_ltax(r)          "Yes if ltax(r) is valid"
    fl_lsub(r)          "Yes if lsub(r) is valid"
;
fl_lump(r) = yes;
fl_ltax(r) = no;
fl_lsub(r) = no;

parameter
    s_f         Scale factor
    s_fac(f)    Scale factor for primary factor
;
s_f = 1;
s_fac(f) = 1;
display s_f, s_fac;

* option sysout = on;

$ontext
$model:iet_lab

*       Debug options for MPSGE
* $echop:.true.
* $datech:.true.
* $funlog:.true.

$sectors:
    u(r)                                ! Utility
    c(r)                                ! Consumption
    gov(r)                              ! Government demand
    inv(r)                              ! Investment
    y(i,r)$vom(i,r)                     ! Supply
    m(i,r)$vim(i,r)                     ! Imports
    yaf(i,j,r)$(vafm(i,j,r)$(not crp(j))) ! Armington intermediate inputs
    yfd(en,j,r)$(vdfm(en,j,r)$(not crp(j))) ! Domestic goods for intermediate inputs
    yfi(en,j,r)$(vifm(en,j,r)$(not crp(j))) ! Import goods for intermediate inputs
    yafc(i,crp,r)$vafm_c(i,crp,r)       ! Armington intermediate inputs for combustion use in CRP
    yfdc(en,crp,r)$vdfm_c(en,crp,r)     ! Domestic goods for intermediate inputs
    yfic(en,crp,r)$vifm_c(en,crp,r)     ! Import goods for intermediate inputs
    yafnc(en,crp,r)$vafm_nc(en,crp,r)   ! Armington intermediate inputs for non-combustion use in CRP
    yap(i,r)$vapm(i,r)                  ! Armington private expenditures
    ypd(en,r)$vdpm(en,r)                ! Domestic goods for consumption
    ypi(en,r)$vipm(en,r)                ! Import goods for consumption
    yag(i,r)$vagm(i,r)                  ! Armington government expenditures
    ygd(en,r)$vdgm(en,r)                ! Domestic goods for government expenditures
    ygi(en,r)$vigm(en,r)                ! Import goods for government expenditures
    yai(i,r)$vaim(i,r)                  ! Armington investment expenditures
    yt(j)$vtw(j)                        ! Transportation services
    ft(f,r)$(sf(f) and evom(f,r))       ! Specific factor transformation
    ls(r)                               ! Labor supply

$commodities:
    pu(r)                               ! Price index of utility
    pc(r)                               ! Private consumption price index
    pg(r)                               ! Public consumption price index
    pinv(r)                             ! Price index of investment
    py(j,r)$vom(j,r)                    ! Domestic output price
    pm(j,r)$vim(j,r)                    ! Import price
    pyaf(i,j,r)$(vafm(i,j,r)$(not crp(j))) ! INT Armington composite price
    pyfd(en,j,r)$(vdfm(en,j,r)$(not crp(j)))
    pyfi(en,j,r)$(vifm(en,j,r)$(not crp(j)))
    pyafc(i,crp,r)$vafm_c(i,crp,r)      ! INT Armington composite price
    pyfdc(en,crp,r)$vdfm_c(en,crp,r)
    pyfic(en,crp,r)$vifm_c(en,crp,r)
    pyafnc(en,crp,r)$vafm_nc(en,crp,r)  ! INT Armington composite price
    pyap(i,r)$vapm(i,r)                 ! HH Armington composite price
    pypd(en,r)$vdpm(en,r)
    pypi(en,r)$vipm(en,r)
    pyag(i,r)$vagm(i,r)                 ! HH Armington composite price
    pygd(en,r)$vdgm(en,r)
    pygi(en,r)$vigm(en,r)
    pyai(i,r)$vaim(i,r)                 ! HH Armington composite price
    pt(j)$vtw(j)                        ! Transportation services
    pf(f,r)$(sf(f) and evom(f,r))       ! Primary factors rent
    pl(r)                               ! wage rate for producer
    plh(r)                              ! wage rate for HH
    ps(f,j,r)$(sf(f) and vfm(f,j,r))    ! Sector-specific primary factors
    rkr(r)$rsk                          ! Return to regional capital
    rkg$gk                              ! Return to global capital
    pco2(r)$pntr(r)                     ! Carbon permits -- non-tradable
    pco2w$ptr_p                         ! Carbon permits -- tradable

$consumers:
    m_h(r)                              ! Representative household
    m_g(r)                              ! Representative government

$auxiliary:
    lump(r)$fl_lump(r)                  ! Lump sum transfer
    ltax(r)$fl_ltax(r)                  ! Labor tax
    esl(r)$(fl_mw or fl_wc)             ! Excess supply of labor
    ur(r)$fl_wc                         ! Unemployment rate
    lsub(r)$fl_lsub(r)                  ! Labor subisidy (* -1)

*       ----------------------------------------------------------------------
*       Production functions:

*       Non-fossil fuel production:
$prod:y(j,r)$oth(j,r)  s:0  vae(s):esubvae(j)  va(vae):esubva(j)
+       e(vae):esube(j)  nel(e):esubnel(j)  lqd(nel):esublqd(j)
    o:py(j,r)      q:vom(j,r)     a:m_g(r)  t:rto(j,r)
    i:ps(sf,j,r)   q:vfm(sf,j,r)  p:(1+rtf0(sf,j,r))  va:
+                                 a:m_g(r)  t:rtf(sf,j,r)
    i:pl(r)        q:vld(j,r)     p:1  va:
+   a:m_g(r)    n:lsub(r)$fl_lsub(r)  m:(-1)$fl_lsub(r)
    i:rkr(r)$rsk   q:vfm("cap",j,r)  p:(1+rtf0("cap",j,r))  va:
+                                    a:m_g(r)  t:rtf("cap",j,r)
    i:rkg$gk       q:vfm("cap",j,r)  p:(1+rtf0("cap",j,r))  va:
+                                    a:m_g(r) t:rtf("cap",j,r)
    i:pyaf(i,j,r)$(not fe(i))   q:vafm(i,j,r)      e:$ele(i)
    i:pyaf("col",j,r)           q:vafm("col",j,r)  nel:
    i:pyaf(i,j,r)$lqd(i)        q:vafm(i,j,r)      lqd:

*       Petroleum and coal products (refined) production (oil only):
$prod:y(j,r)$(vom(j,r) and oil(j))  s:0  vae(s):esubvae(j)  va(vae):esubva(j)  e(vae):esube(j)
    o:py(j,r)      q:vom(j,r)     a:m_g(r)  t:rto(j,r)
    i:pyaf(i,j,r)  q:vafm(i,j,r)  e:$(gas(i) or ele(i))
    i:ps(sf,j,r)   q:vfm(sf,j,r)  p:(1+rtf0(sf,j,r))  va:  a:m_g(r)  t:rtf(sf,j,r)
    i:pl(r)     q:vld(j,r)  p:1  va:
+   a:m_g(r)    n:lsub(r)$fl_lsub(r)  m:(-1)$fl_lsub(r)
    i:rkr(r)$rsk   q:vfm("cap",j,r)  p:(1+rtf0("cap",j,r))  va:  a:m_g(r)  t:rtf("cap",j,r)
    i:rkg$gk       q:vfm("cap",j,r)  p:(1+rtf0("cap",j,r))  va:  a:m_g(r)  t:rtf("cap",j,r)

*       Chemicals production (crp):
$prod:y(crp,r)$vom(crp,r)  s:0  vae(s):esubvae(crp)  va(vae):esubva(crp)
+                      e(vae):esube(crp)  nel(e):esubnel(crp)  lqd(nel):esublqd(crp)
    o:py(crp,r)    q:vom(crp,r)     a:m_g(r)  t:rto(crp,r)
    i:pyafnc(en,crp,r)   q:vafm_nc(en,crp,r)
    i:pyafc(i,crp,r)$(not fe(i))  q:vafm_c(i,crp,r) e:$ele(i)
    i:pyafc("col",crp,r)    q:vafm_c("col",crp,r)  nel:
    i:pyafc("oil",crp,r)    q:vafm_c("oil",crp,r)  lqd:
    i:pyafc("gas",crp,r)    q:vafm_c("gas",crp,r)  lqd:
    i:ps(sf,crp,r)   q:vfm(sf,crp,r)  p:(1+rtf0(sf,crp,r)) va:
+                                     a:m_g(r) t:rtf(sf,crp,r)
    i:pl(r)     q:vld(crp,r)  p:1  va:
+   a:m_g(r)    n:lsub(r)$fl_lsub(r)  m:(-1)$fl_lsub(r)
    i:rkr(r)$rsk   q:vfm("cap",crp,r)   p:(1+rtf0("cap",crp,r)) va:
+                                                 a:m_g(r) t:rtf("cap",crp,r)
    i:rkg$gk       q:vfm("cap",crp,r)   p:(1+rtf0("cap",crp,r)) va:
+                                                 a:m_g(r) t:rtf("cap",crp,r)

*       Fossil fuel production (col, cru, gas):
$prod:y(xe,r)$vom(xe,r)  s:(esub_es(xe,r))  id:0
    o:py(xe,r)          q:vom(xe,r)  a:m_g(r)  t:rto(xe,r)
    i:pyaf(i,xe,r)      q:vafm(i,xe,r)  id:
    i:pl(r)             q:vld(xe,r)  p:1  id:
+   a:m_g(r)    n:lsub(r)$fl_lsub(r)  m:(-1)$fl_lsub(r)
    i:rkr(r)$rsk        q:vfm("cap",xe,r)  p:(1+rtf0("cap",xe,r))  id:
+                       a:m_g(r)  t:rtf("cap",xe,r)
    i:rkg$gk            q:vfm("cap",xe,r)  p:(1+rtf0("cap",xe,r))  id:
+                       a:m_g(r)  t:rtf("cap",xe,r)
    i:ps("lnd",xe,r)    q:vfm("lnd",xe,r)  p:(1+rtf0("lnd",xe,r))  id:
+                                           a:m_g(r)  t:rtf("lnd",xe,r)
    i:ps("nrs",xe,r)    q:vfm("nrs",xe,r)  p:(1+rtf0("nrs",xe,r))
+                                           a:m_g(r)  t:rtf("nrs",xe,r)

*       ----------------------------------------------------------------------
*       Armington aggregation over domestic versus imports:

*       Armington aggregation (non-energy goods)
$prod:yaf(i,ncrp,r)$(vafm(i,ncrp,r)$(not en(i)))  s:esubd(i)
    o:pyaf(i,ncrp,r)  q:vafm(i,ncrp,r)
    i:py(i,r)  q:vdfm(i,ncrp,r)  p:(1+rtfd0(i,ncrp,r))  a:m_g(r)  t:rtfd(i,ncrp,r)
    i:pm(i,r)  q:vifm(i,ncrp,r)  p:(1+rtfi0(i,ncrp,r))  a:m_g(r)  t:rtfi(i,ncrp,r)

*       Armington aggregation (energy goods)
$prod:yaf(en,j,r)$(vafm(en,j,r)$(not crp(j)))  s:esubd(en)
    o:pyaf(en,j,r)  q:vafm(en,j,r)
    i:pyfd(en,j,r)  q:vdfm(en,j,r)  p:(1+rtfd0(en,j,r))  a:m_g(r) t:rtfd(en,j,r)
    i:pyfi(en,j,r)  q:vifm(en,j,r)  p:(1+rtfi0(en,j,r))  a:m_g(r) t:rtfi(en,j,r)

*       Energy goods (domestic)
$prod:yfd(en,j,r)$(vdfm(en,j,r)$(not crp(j)))  s:0
    o:pyfd(en,j,r) q:vdfm(en,j,r)
    i:py(en,r)     q:vdfm(en,j,r)
    i:pco2(r)$pntr(r)  q:co2fd(en,j,r)  p:1e-6
    i:pco2w$ptr(r)     q:co2fd(en,j,r)  p:1e-6

*       Energy goods (import)
$prod:yfi(en,j,r)$(vifm(en,j,r)$(not crp(j)))  s:0
    o:pyfi(en,j,r) q:vifm(en,j,r)
    i:pm(en,r)     q:vifm(en,j,r)
    i:pco2(r)$pntr(r)  q:co2fi(en,j,r)  p:1e-6
    i:pco2w$ptr(r)     q:co2fi(en,j,r)  p:1e-6

*       Armington aggregation for CRP sector (non-energy goods):
$prod:yafc(i,crp,r)$(vafm_c(i,crp,r)$(not en(i)))  s:esubd(i)
    o:pyafc(i,crp,r)  q:vafm_c(i,crp,r)
    i:py(i,r)  q:vdfm_c(i,crp,r)  p:(1+rtfd0(i,crp,r))  a:m_g(r)  t:rtfd(i,crp,r)
    i:pm(i,r)  q:vifm_c(i,crp,r)  p:(1+rtfi0(i,crp,r))  a:m_g(r)  t:rtfi(i,crp,r)

*       Armington aggregation for CRP sector (energy goods):
$prod:yafc(en,crp,r)$vafm_c(en,crp,r)  s:esubd(en)
    o:pyafc(en,crp,r)  q:vafm_c(en,crp,r)
    i:pyfdc(en,crp,r)  q:vdfm_c(en,crp,r)  p:(1+rtfd0(en,crp,r))  a:m_g(r)  t:rtfd(en,crp,r)
    i:pyfic(en,crp,r)  q:vifm_c(en,crp,r)  p:(1+rtfi0(en,crp,r))  a:m_g(r)  t:rtfi(en,crp,r)

*       Energy goods (domestic)
$prod:yfdc(en,crp,r)$vdfm_c(en,crp,r)  s:0
    o:pyfdc(en,crp,r)  q:vdfm_c(en,crp,r)
    i:py(en,r)         q:vdfm_c(en,crp,r)
    i:pco2(r)$pntr(r)  q:co2fd(en,crp,r)  p:1e-6
    i:pco2w$ptr(r)     q:co2fd(en,crp,r)  p:1e-6

*       Energy goods (import)
$prod:yfic(en,crp,r)$vifm_c(en,crp,r)  s:0
    o:pyfic(en,crp,r)  q:vifm_c(en,crp,r)
    i:pm(en,r)         q:vifm_c(en,crp,r)
    i:pco2(r)$pntr(r)  q:co2fi(en,crp,r)  p:1e-6
    i:pco2w$ptr(r)     q:co2fi(en,crp,r)  p:1e-6

*       Armington aggregation for CRP sector (non-combustion use):
$prod:yafnc(en,crp,r)$vafm_nc(en,crp,r)  s:esubd(en)
    o:pyafnc(en,crp,r) q:vafm_nc(en,crp,r)
    i:py(en,r)  q:vdfm_nc(en,crp,r)  p:(1+rtfd0(en,crp,r))  a:m_g(r) t:rtfd(en,crp,r)
    i:pm(en,r)  q:vifm_nc(en,crp,r)  p:(1+rtfi0(en,crp,r))  a:m_g(r) t:rtfi(en,crp,r)

*       ----------------------------------------------------------------------
*       International transport services (Cobb-Douglas):
$prod:yt(j)$vtw(j)  s:1
    o:pt(j)       q:vtw(j)
    i:py(j,r)     q:vst(j,r)

*       ----------------------------------------------------------------------
*       Labor supply
$prod:ls(r)  s:0
    o:pl(r)   q:vls(r)  p:1  a:m_g(r)  t:ltax0(r)$(not fl_ltax(r))  n:ltax(r)$fl_ltax(r)
    i:plh(r)  q:vls(r)  p:plh0(r)

*       ----------------------------------------------------------------------
*       Final demand:

*       Utility:
$prod:u(r)  s:esub_lsr(r)
    o:pu(r)     q:vu(r)
    i:pc(r)     q:vc(r)
    i:plh(r)$fl_lei  q:vlsr(r)  p:plh0(r)

*       Consumption:
$prod:c(r)  s:esubc  c(s):esubcc  e(s):esubec
    o:pc(r)      q:vc(r)
    i:pyap(i,r)  q:vapm(i,r)  c:$(not e(i))  e:$e(i)

*       Armington aggregation for private expenditure (non-energy goods):
$prod:yap(i,r)$(vapm(i,r)$(not en(i)))  s:esubd(i)
    o:pyap(i,r)  q:vapm(i,r)
    i:py(i,r)    q:vdpm(i,r)  p:(1+rtpd0(i,r))  a:m_g(r)  t:rtpd(i,r)
    i:pm(i,r)    q:vipm(i,r)  p:(1+rtpi0(i,r))  a:m_g(r)  t:rtpi(i,r)

*       Armington aggregation for private expenditure (energy goods):
$prod:yap(en,r)$vapm(en,r)  s:esubd(en)
    o:pyap(en,r)  q:vapm(en,r)
    i:pypd(en,r)  q:vdpm(en,r)  p:(1+rtpd0(en,r))  a:m_g(r)  t:rtpd(en,r)
    i:pypi(en,r)  q:vipm(en,r)  p:(1+rtpi0(en,r))  a:m_g(r)  t:rtpi(en,r)

*       Armington aggregation for private expenditure:
$prod:ypd(en,r)$vdpm(en,r)  s:0
    o:pypd(en,r)  q:vdpm(en,r)
    i:py(en,r)    q:vdpm(en,r)
    i:pco2(r)$pntr(r)  q:co2hd(en,r)  p:1e-6
    i:pco2w$ptr(r)     q:co2hd(en,r)  p:1e-6

$prod:ypi(en,r)$vipm(en,r)  s:0
    o:pypi(en,r)  q:vipm(en,r)
    i:pm(en,r)    q:vipm(en,r)
    i:pco2(r)$pntr(r)  q:co2hi(en,r)  p:1e-6
    i:pco2w$ptr(r)     q:co2hi(en,r)  p:1e-6

*       ----------------------------------------------------------------------
*       Public consumption:
$prod:gov(r)  s:0
    o:pg(r)      q:vgm(r)
    i:pyag(i,r)  q:vagm(i,r)

*       Armington aggregation for government expenditure (non-energy)
$prod:yag(i,r)$(vagm(i,r)$(not en(i)))  s:esubd(i)
    o:pyag(i,r)  q:vagm(i,r)
    i:py(i,r)    q:vdgm(i,r)  p:(1+rtgd0(i,r))  a:m_g(r)  t:rtgd(i,r)
    i:pm(i,r)    q:vigm(i,r)  p:(1+rtgi0(i,r))  a:m_g(r)  t:rtgi(i,r)

*       Armington aggregation for government expenditure (energy)
$prod:yag(en,r)$vagm(en,r)  s:esubd(en)
    o:pyag(en,r)  q:vagm(en,r)
    i:pygd(en,r)  q:vdgm(en,r)  p:(1+rtgd0(en,r))  a:m_g(r)  t:rtgd(en,r)
    i:pygi(en,r)  q:vigm(en,r)  p:(1+rtgd0(en,r))  a:m_g(r)  t:rtgi(en,r)

*       Energy goods (domestic)
$prod:ygd(en,r)$vdgm(en,r)  s:0
    o:pygd(en,r)  q:vdgm(en,r)
    i:py(en,r)    q:vdgm(en,r)

*       Energy goods (import)
$prod:ygi(en,r)$vigm(en,r)  s:0
    o:pygi(en,r)  q:vigm(en,r)
    i:pm(en,r)    q:vigm(en,r)

*       ----------------------------------------------------------------------
*       Armington aggregation across imports from different countries:
$prod:m(i,r)$vim(i,r)  s:esubm(i)  s.tl:0
    o:pm(i,r)   q:vim(i,r)
    i:py(i,s)   q:vxmd(i,s,r)  p:pvxmd(i,s,r)  s.tl:
+               a:m_g(s)  t:(-rtxs(i,s,r))
+               a:m_g(r)  t:(rtms(i,s,r)*(1-rtxs(i,s,r)))
    i:pt(j)#(s)           q:vtwr(j,i,s,r)  p:pvtwr(i,s,r)  s.tl:
+               a:m_g(r)  t:rtms(i,s,r)

*       ----------------------------------------------------------------------
*       Sluggish Factor Transformation:
$prod:ft(sf,r)$evom(sf,r)  t:etrae(sf)
    o:ps(sf,j,r)  q:vfm(sf,j,r)
    i:pf(sf,r)    q:evom(sf,r)

*       ----------------------------------------------------------------------
*       Investment
$prod:inv(r)  s:0
    o:pinv(r)    q:vinv(r)
    i:pyai(i,r)  q:vaim(i,r)

$prod:yai(i,r)$vaim(i,r)  s:esubd(i)
    o:pyai(i,r)  q:vaim(i,r)
    i:py(i,r)    q:vdim(i,r)  p:(1+rtid0(i,r))  a:m_g(r)  t:rtid(i,r)
    i:pm(i,r)    q:viim(i,r)  p:(1+rtii0(i,r))  a:m_g(r)  t:rtii(i,r)

*       ----------------------------------------------------------------------
*       Privagte household (household income balance):
$demand:m_h(r)
    d:pu(r)             q:vu(r)
    e:pinv(r)           q:(-s_f*vinv(r))
    e:pf(f,r)$sf(f)     q:(s_f*s_fac(f)*evom(f,r))
    e:plh(r)            q:(s_f*s_fac("lab")*l_bar(r))
    e:plh(r)$(fl_mw or fl_wc)  q:(-1)  r:esl(r)
    e:rkr(r)$rsk        q:(s_f*s_fac("cap")*evom("cap",r))
    e:rkg$gk            q:(s_f*s_fac("cap")*evom("cap",r))
    e:pc(r)             q:(-s_f*vtax(r))
    e:pc(rnum)          q:(s_f*vb(r))
    e:pc(r)$fl_lump(r)  q:1     r:lump(r)

*       ----------------------------------------------------------------------
*       Government (public budget):
$demand:m_g(r)
    d:pg(r)             q:vgm(r)
    e:pc(r)             q:(s_f*vtax(r))
    e:pco2(r)$pntr(r)   q:co2lim(r)
    e:pco2w$ptr(r)      q:co2lim(r)
    e:pc(r)$fl_lump(r)  q:(-1)  r:lump(r)

*       ----------------------------------------------------------------------
*       lump sum tax & transfer:
$constraint:lump(r)$fl_lump(r)
    m_g(r) =g= pg(r) * vgm(r) * s_f;

*       ----------------------------------------------------------------------
*       Tax on labour:
$constraint:ltax(r)$fl_ltax(r)
    m_g(r) =e= pg(r) * vgm(r) * s_f;

*       ----------------------------------------------------------------------
*       Constraint on real wage.
$constraint:ur(r)$fl_wc
    pl(r) / pu(r) =e= v0(r) * (ur(r) / ur0(r))**(wc_elas);

*       Excess supply of labor.
$constraint:esl(r)$(fl_mw or fl_wc)
    (pl(r) - pu(r) * v0(r))$fl_mw + (esl(r) - ur(r) * (l_bar(r)*s_f))$fl_wc =g= 0;

*       Labor subsidy
$constraint:lsub(r)$fl_lsub(r)
    (ur0(r) - ur(r))$fl_wc + (esl(r) - unempr0(r))$fl_mw =g= 0;

*       ----------------------------------------------------------------------
*       Variables for reporting:
$report:
    v:z_y(j,r)$vom(j,r)           o:py(j,r)    prod:y(j,r)      ! Output
    v:z_fy(f,j,r)$(not k(f))      i:pl(r)      prod:y(j,r)      ! Factor demand
    v:z_fy(f,j,r)$(rsk and k(f))  i:rkr(r)     prod:y(j,r)      ! Factor demand
    v:z_fy(f,j,r)$(gk and k(f))   i:rkg        prod:y(j,r)      ! Factor demand
    v:z_sy(f,j,r)                 i:ps(f,j,r)  prod:y(j,r)      ! Factor demand
    v:z_yaf(i,j,r)$(vafm(i,j,r) and (not crp(j)))    i:pyaf(i,j,r)  prod:y(j,r) ! Intermediate demand
    v:z_yafc(i,crp,r)$vafm_c(i,crp,r)    i:pyafc(i,crp,r)   prod:y(crp,r)       ! Intermediate demand
    v:z_yafnc(i,crp,r)$vafm_nc(i,crp,r)  i:pyafnc(i,crp,r)  prod:y(crp,r)       ! Intermediate demand

$report:
    v:z_yap(i,r)$vapm(i,r)  i:pyap(i,r)  prod:c(r)      ! Consumption demand
    v:z_yag(i,r)$vagm(i,r)  i:pyag(i,r)  prod:gov(r)    ! Government demand
    v:z_yai(i,r)$vaim(i,r)  i:pyai(i,r)  prod:inv(r)    ! Investment demand

$report:
    v:z_qco2fd(en,j,r)$(pntr(r) and co2fd(en,j,r)) i:pco2(r)  prod:yfd(en,j,r)
    v:z_qco2fi(en,j,r)$(pntr(r) and co2fi(en,j,r)) i:pco2(r)  prod:yfi(en,j,r)
    v:z_qco2fd(en,j,r)$(ptr(r) and co2fd(en,j,r))  i:pco2w  prod:yfd(en,j,r)
    v:z_qco2fi(en,j,r)$(ptr(r) and co2fi(en,j,r))  i:pco2w  prod:yfi(en,j,r)
    v:z_qco2fdc(en,crp,r)$(pntr(r) and co2fd(en,crp,r))  i:pco2(r)  prod:yfdc(en,crp,r)
    v:z_qco2fic(en,crp,r)$(pntr(r) and co2fi(en,crp,r))  i:pco2(r)  prod:yfic(en,crp,r)
    v:z_qco2fdc(en,crp,r)$(ptr(r) and co2fd(en,crp,r))  i:pco2w  prod:yfdc(en,crp,r)
    v:z_qco2fic(en,crp,r)$(ptr(r) and co2fi(en,crp,r))  i:pco2w  prod:yfic(en,crp,r)
    v:z_qco2hd(en,r)$(pntr(r) and co2hd(en,r))  i:pco2(r)  prod:ypd(en,r)
    v:z_qco2hi(en,r)$(pntr(r) and co2hi(en,r))  i:pco2(r)  prod:ypi(en,r)
    v:z_qco2hd(en,r)$(ptr(r) and co2hd(en,r))  i:pco2w  prod:ypd(en,r)
    v:z_qco2hi(en,r)$(ptr(r) and co2hi(en,r))  i:pco2w  prod:ypi(en,r)

$report:
    v:z_con(r)  o:pc(r)    prod:c(r)                    ! Consumption
    v:z_inv(r)  o:pinv(r)  prod:inv(r)                  ! Investment
    v:z_gov(r)  o:pg(r)    prod:gov(r)                  ! Government expenditure
    v:z_imp(i,r)$vim(i,r)  o:pm(i,r)  prod:m(i,r)       ! Import
    v:z_exp(i,s,r)$vxmd(i,s,r)  i:py(i,s)  prod:m(i,r)  ! Export

$report:
    v:z_vst(j,r)$vst(j,r)  i:py(j,r)  prod:yt(j)        ! Supply to international trans sec
    v:z_lsrd(r)$fl_lei     i:pl(r)    prod:u(r)         ! Leisure demand
    v:z_ls(r)              i:plh(r)   prod:ls(r)        ! Labor supply

$report:
    v:z_yfd(i,j,r)$(vdfm(i,j,r) and (not en(i)) and (not crp(j)))  i:py(i,r)  prod:yaf(i,j,r)   ! Intermediate demand (dom)
    v:z_yfi(i,j,r)$(vifm(i,j,r) and (not en(i)) and (not crp(j)))  i:pm(i,r)  prod:yaf(i,j,r)   ! Intermediate demand (imp)
    v:z_yfd(i,j,r)$(vdfm(i,j,r) and en(i) and (not crp(j)))  i:pyfd(i,j,r)  prod:yaf(i,j,r)     ! Intermediate inputs (dom)
    v:z_yfi(i,j,r)$(vifm(i,j,r) and en(i) and (not crp(j)))  i:pyfi(i,j,r)  prod:yaf(i,j,r)     ! Intermediate inputs (imp)
    v:z_yfcd(i,crp,r)$(vdfm_c(i,crp,r) and (not en(i)))  i:py(i,r)  prod:yafc(i,crp,r)          ! Intermediate demand (dom)
    v:z_yfci(i,crp,r)$(vifm_c(i,crp,r) and (not en(i)))  i:pm(i,r)  prod:yafc(i,crp,r)          ! Intermediate demand (imp)
    v:z_yfcd(i,crp,r)$(vdfm_c(i,crp,r) and en(i))  i:pyfdc(i,crp,r)  prod:yafc(i,crp,r)         ! Intermediate demand (dom)
    v:z_yfci(i,crp,r)$(vifm_c(i,crp,r) and en(i))  i:pyfic(i,crp,r)  prod:yafc(i,crp,r)         ! Intermediate demand (imp)
    v:z_yfncd(i,crp,r)$(vdfm_nc(i,crp,r) and en(i))  i:py(i,r)  prod:yafnc(i,crp,r)             ! Intermediate demand (dom)
    v:z_yfnci(i,crp,r)$(vifm_nc(i,crp,r) and en(i))  i:pm(i,r)  prod:yafnc(i,crp,r)             ! Intermediate demand (imp)

$report:
    v:z_ypd(i,r)$(vdpm(i,r) and (not en(i)))  i:py(i,r)  prod:yap(i,r)  ! Consumption demand (dom)
    v:z_ypi(i,r)$(vipm(i,r) and (not en(i)))  i:pm(i,r)  prod:yap(i,r)  ! Consumption demand (imp)
    v:z_ypd(i,r)$(vdpm(i,r) and en(i))  i:pypd(i,r)  prod:yap(i,r)      ! Consumption demand (dom)
    v:z_ypi(i,r)$(vipm(i,r) and en(i))  i:pypi(i,r)  prod:yap(i,r)      ! Consumption demand (imp)
    v:z_ygd(i,r)$(vdgm(i,r) and (not en(i)))  i:py(i,r)  prod:yag(i,r)  ! Government demand (dom)
    v:z_ygi(i,r)$(vigm(i,r) and (not en(i)))  i:pm(i,r)  prod:yag(i,r)  ! Government demand (imp)
    v:z_ygd(i,r)$(vdgm(i,r) and en(i)) i:pygd(i,r)  prod:yag(i,r)       ! Government demand (dom)
    v:z_ygi(i,r)$(vigm(i,r) and en(i)) i:pygi(i,r)  prod:yag(i,r)       ! Government demand (imp)
    v:z_yid(i,r)$vdim(i,r)  i:py(i,r)  prod:yai(i,r)                    ! Investment demand (dom)
    v:z_yii(i,r)$viim(i,r)  i:pm(i,r)  prod:yai(i,r)                    ! Investment demand (imp)

$offtext

$sysinclude mpsgeset iet_lab

*       Numeraire:
m_h.fx(rnum) = vu(rnum);

*       ----------------------------------------------------------------------
*       Initial values for variables:
display "@ Initial values for variables:";

*       Initial wage rate for household:
plh.l(r) = plh0(r);

*       ----------------------------------------------------------------------
*       No carbon dioxide taxes in the benchmark:
co2lim(r) = co2bench(r) * 2;
pco2.l(r) = 0;
pco2w.l =  0;

*       By default, IET is not used.
pntr(r) = 1;
ptr(r) = 0;
ptr_p = 0;

*       Initial values for lump-sum tax variables:
lump.lo(r) = -inf;
lump.up(r) = +inf;
lump.l(r) = 0;

ltax.l(r) = ltax0(r);
ltax.lo(r) = -inf;

esl.l(r) = 0 + unempr0(r)$(fl_mw or fl_wc);
esl.lo(r) = -1e-1;
ur.l(r) = ur0(r);
ur.lo(r) = -1e-6;

lsub.l(r) = 0;
lsub.lo(r) = -1e-1;

*       ----------------------------------------------------------------------
*       Parameters for reporting results:
$include output_par.gms

*       Reduction rate:
* gams-include-file: .\scenarios\rdr_default.gms
$include %rdr_file%
rd_rate(r) = rd_rate(r) * %s_rd%;
display rd_rate;

subj(r) = no;
subj(r)$rd_rate(r) = yes;
display subj;

*       ----------------------------------------------------------------------
*       Benchmark replication.
display "@ Benchmark replication.";

*       ----------------------------------------------------------------------
*       Policy choice for benchmark replication:
fl_lump(r) = yes;

putclose titlemaker '@title Scenario %s_name%: Benchmark replication';
execute 'titlemaker_%s_name%.cmd';

iet_lab.optfile = 1;
iet_lab.workspace = 300;
iet_lab.iterlim = 0;
$include iet_lab.gen

solve iet_lab using mcp;

$if not %br_only%==0 $goto end

*       ----------------------------------------------------------------------
*       Cleanup calculation.
display "@ Cleanup calculation.";

putclose titlemaker '@title Scenario %s_name%: Cleanup calculation';
execute 'titlemaker_%s_name%.cmd';

iet_lab.iterlim = 10000;

$include iet_lab.gen
solve iet_lab using mcp;

$if not %cc_only%==0 $goto end

chk_objval("bau") = iet_lab.objval;
abort$(abs(chk_objval("bau")) > 1e-4) "Objval is too large!!!";
display chk_objval;

loop(psol$pbau(psol),
$include output_results.gms
);

$if %s_name%=="bau" $goto bau

$if %scale_shock%==0 $goto sc_end
*       ----------------------------------------------------------------------
*       Scale shock experiments
display "@ Scale shock experiments:";

putclose titlemaker '@title Scenario %s_name%: Scale shock %scale_shock% increase.';
execute 'titlemaker_%s_name%.cmd';

s_f = 1 + (%scale_shock%) / 100;
$include iet_lab.gen
solve iet_lab using mcp;
chk_objval(psol) = iet_lab.objval;
display chk_objval;
s_f = 1;

$if not %scale_shock%==0 $goto end
$label sc_end

*       ----------------------------------------------------------------------
*       Carbon regulation:
display "@ Carbon regulation:";

co2lim(r) = co2bench(r) * 2;
co2lim(r)$rd_rate(r) = co2bench(r) * (1 + rd_rate(r)/100);
display co2lim;

*       Policy choice
fl_lump(r) = no;
fl_ltax(r) = no;
fl_lsub(r) = no;
* gams-include-file: .\scenarios\pol_default.gms
$include %pol_file%

display fl_lump, fl_ltax, fl_lsub, cap;

loop(psol$(not pbau(psol)),

    putclose titlemaker '@title Scenario %s_name%: Carbon regulation, pattern = ', psol.tl;
    execute 'titlemaker_%s_name%.cmd';

    pcur(pall) = no;
    pcur(psol) = yes;

*       Settings for permit trade:
* gams-include-file: .\scenarios\ptrade_default.gms
$include %ptrade_file%
    ptr_p$sum(r$ptr(r), 1) = 1;
    display pntr, ptr, ptr_p;

*       Solve the model:
$include iet_lab.gen
    solve iet_lab using mcp;

    chk_objval(psol) = iet_lab.objval;
    abort$(abs(chk_objval(psol)) > 1e-4) "Objval is too large!!!";
    display chk_objval;

$include output_results.gms

);

$label report

display chk_objval
display l_u, l_lsrd, l_ls, l_c, l_inv, l_gov, l_exp, l_imp, l_gdp, l_exp_c,
        l_imp_c, l_pco2, l_pco2_, l_y, l_ld, l_kd, l_vperm, l_vpermn, l_labinc,
        l_labtax, l_tot;

display l_co2, l_co2_so, l_co2_use, l_co2_t, l_ptrade, l_vptrade;

*       ----------------------------------------------------------------------
*       Leakage rate:

*       Carbon leakage by sector:
d_co2_leak_sec(i,"MtCO2",psol_) =
    l_co2_leak_sec(psol_,i,"nsubj") - l_co2_leak_sec("bau",i,"nsubj");
d_co2_leak_sec(ssec,"MtCO2",psol_) =
    l_co2_leak_sec(psol_,ssec,"nsubj") - l_co2_leak_sec("bau",ssec,"nsubj");

*       Carbon leakage by region
d_co2_leak_reg(r,"MtCO2",psol_) = l_co2_t(r,psol_) - l_co2_t(r,"bau");
d_co2_leak_reg("nsubj","MtCO2",psol_) = sum(r$(not subj(r)), d_co2_leak_reg(r,"MtCO2",psol_));
d_co2_leak_reg("subj","MtCO2",psol_) = sum(r$subj(r), d_co2_leak_reg(r,"MtCO2",psol_));
d_co2_leak_reg("sum","MtCO2",psol_) = sum(r, d_co2_leak_reg(r,"MtCO2",psol_));

display d_co2_leak_reg, l_co2_t;

*       Plus:
d_co2_leak("nsubj",psol_) = l_co2_leak(psol_,"nsubj") - l_co2_leak("bau","nsubj");
*       Minus:
d_co2_leak("subj",psol_) = l_co2_leak(psol_,"subj") - l_co2_leak("bau","subj");
d_co2_leak("sum",psol_) = d_co2_leak("subj",psol_) + d_co2_leak("nsubj",psol_);

display l_co2_leak, d_co2_leak;

*       Leakage rate by region (%):
l_leak_rate_reg(r,"%",psol_)$(not subj(r))
    = 100 * d_co2_leak_reg(r,"MtCO2",psol_)
    / sum(rr$subj(rr), - d_co2_leak_reg(rr,"MtCO2",psol_));
l_leak_rate_reg("sum","%",psol_) = sum(r$(not subj(r)), l_leak_rate_reg(r,"%",psol_));

*       Total leakage rate (%):
l_leak_rate("Leakage_rate (%)","%",psol_)
    = - 100 * (d_co2_leak("nsubj",psol_) / d_co2_leak("subj",psol_));

*       ----------------------------------------------------------------------
*       Change

*       Change in net export by commodity:
d_nexp_c(i,r,psol_) = l_nexp_c(psol_,i,r) - l_nexp_c("bau",i,r);
d_nexp_c(ssec,r,psol_) = l_nexp_c(psol_,ssec,r) - l_nexp_c("bau",ssec,r);

d_y(i,r,psol_)$l_y("bau",i,r) = l_y(psol_,i,r) - l_y("bau",i,r);
d_y(ssec,r,psol_)$l_y("bau",ssec,r) = l_y(psol_,ssec,r) - l_y("bau",ssec,r);

d_co2_use(i,r,psol_) = l_co2_use(psol_,i,r) - l_co2_use("bau",i,r);
d_co2_use(ssec,r,psol_) = l_co2_use(psol_,ssec,r) - l_co2_use("bau",ssec,r);

d_co2(r,"MtCO2",psol_) = l_co2_t(r,psol_) - l_co2_t(r,"bau");
d_co2("nsubj","MtCO2",psol_) = sum(r$(not subj(r)), d_co2(r,"MtCO2",psol_));
d_co2("subj","MtCO2",psol_) = sum(r$subj(r), d_co2(r,"MtCO2",psol_));
d_co2("sum","MtCO2",psol_) = sum(r, d_co2(r,"MtCO2",psol_));

*       ----------------------------------------------------------------------
*       % change:
pc_u(r,psol_)$l_u("bau",r) = 100 * (l_u(psol_,r) / l_u("bau",r) - 1);
pc_u("subj",psol_)$l_u("bau","subj") = 100 * (l_u(psol_,"subj") / l_u("bau","subj") - 1);
pc_u("nsubj",psol_)$l_u("bau","nsubj") = 100 * (l_u(psol_,"nsubj") / l_u("bau","nsubj") - 1);
pc_u("sum",psol_)$l_u("bau","sum") = 100 * (l_u(psol_,"sum") / l_u("bau","sum") - 1);

pc_lsrd(r,psol_)$l_lsrd("bau",r) = 100 * (l_lsrd(psol_,r) / l_lsrd("bau",r) - 1);
pc_ls(r,psol_)$l_ls("bau",r) = 100 * (l_ls(psol_,r) / l_ls("bau",r) - 1);
pc_c(r,psol_)$l_c("bau",r) = 100 * (l_c(psol_,r) / l_c("bau",r) - 1);
pc_inv(r,psol_)$l_inv("bau",r) = 100 * (l_inv(psol_,r) / l_inv("bau",r) - 1);
pc_gov(r,psol_)$l_gov("bau",r) = 100 * (l_gov(psol_,r) / l_gov("bau",r) - 1);
pc_exp(r,psol_)$l_exp("bau",r) = 100 * (l_exp(psol_,r) / l_exp("bau",r) - 1);
pc_imp(r,psol_)$l_imp("bau",r) = 100 * (l_imp(psol_,r) / l_imp("bau",r) - 1);

pc_gdp(r,psol_)$l_gdp("bau",r) = 100 * (l_gdp(psol_,r) / l_gdp("bau",r) - 1);
pc_gdp("subj",psol_)$l_gdp("bau","subj") = 100 * (l_gdp(psol_,"subj") / l_gdp("bau","subj") - 1);
pc_gdp("nsubj",psol_)$l_gdp("bau","nsubj") = 100 * (l_gdp(psol_,"nsubj") / l_gdp("bau","nsubj") - 1);
pc_gdp("sum",psol_)$l_gdp("bau","sum") = 100 * (l_gdp(psol_,"sum") / l_gdp("bau","sum") - 1);

pc_exp_c(i,r,psol_)$(subj(r) and l_exp_c("bau",i,r))
    = 100 * (l_exp_c(psol_,i,r) / l_exp_c("bau",i,r) - 1);
pc_imp_c(i,r,psol_)$(subj(r) and l_imp_c("bau",i,r))
    = 100 * (l_imp_c(psol_,i,r) / l_imp_c("bau",i,r) - 1);
pc_nexp_c(i,r,psol_)$(subj(r) and l_nexp_c("bau",i,r))
    = 100 * (l_nexp_c(psol_,i,r) / l_nexp_c("bau",i,r) - 1);
pc_nexp_c("sum",r,psol_)$(subj(r) and l_nexp_c("bau","sum",r))
    = 100 * (l_nexp_c(psol_,"sum",r) / l_nexp_c("bau","sum",r) - 1);

pc_y(i,r,psol_)$l_y("bau",i,r)
    = 100 * (l_y(psol_,i,r) / l_y("bau",i,r) - 1);
pc_y(ssec,r,psol_)$l_y("bau",ssec,r)
    = 100 * (l_y(psol_,ssec,r) / l_y("bau",ssec,r) - 1);

pc_va_s(i,r,psol_)$(subj(r) and l_va_s("bau",i,r))
    = 100 * (l_va_s(psol_,i,r) / l_va_s("bau",i,r) - 1);

pc_ld(i,r,psol_)$(subj(r) and l_ld("bau",i,r))
    = 100 * (l_ld(psol_,i,r) / l_ld("bau",i,r) - 1);
pc_ld(ssec,r,psol_)$(subj(r) and l_ld("bau",ssec,r))
    = 100 * (l_ld(psol_,ssec,r) / l_ld("bau",ssec,r) - 1);

pc_kd(i,r,psol_)$(subj(r) and l_kd("bau",i,r))
    = 100 * (l_kd(psol_,i,r) / l_kd("bau",i,r) - 1);
pc_kd("sum",r,psol_)$(subj(r) and l_kd("bau","sum",r))
    = 100 * (l_kd(psol_,"sum",r) / l_kd("bau","sum",r) - 1);

pc_pl(r,psol_)$l_pl("bau",r) = 100 * (l_pl(psol_,r) / l_pl("bau",r) - 1);
pc_plh(r,psol_)$l_plh("bau",r) = 100 * (l_plh(psol_,r) / l_plh("bau",r) - 1);
pc_rk(r,psol_)$l_rk("bau",r) = 100 * (l_rk(psol_,r) / l_rk("bau",r) - 1);
pc_pya(i,r,psol_)$l_pya("bau",i,r) = 100 * (l_pya(psol_,i,r) / l_pya("bau",i,r) - 1);
pc_pym(i,r,psol_)$l_pym("bau",i,r) = 100 * (l_pym(psol_,i,r) / l_pym("bau",i,r) - 1);
pc_labinc(r,psol_)$l_labinc("bau",r) = 100 * (l_labinc(psol_,r) / l_labinc("bau",r) - 1);
pc_inc(r,psol_)$l_inc("bau",r) = 100 * (l_inc(psol_,r) / l_inc("bau",r) - 1);
pc_pene_ex(i,r,psol_)$l_pene_ex("bau",i,r) = 100 * (l_pene_ex(psol_,i,r) / l_pene_ex("bau",i,r) - 1);

pc_co2(en,j,r,psol_)$l_co2("bau",en,j,r)
    = 100 * (l_co2(psol_,en,j,r) / l_co2("bau",en,j,r) - 1);

pc_co2_use(j,r,psol_)$l_co2_use("bau",j,r)
    = 100 * (l_co2_use(psol_,j,r) / l_co2_use("bau",j,r) - 1);
pc_co2_use("hh",r,psol_)$l_co2_use("bau","hh",r)
    = 100 * (l_co2_use(psol_,"hh",r) / l_co2_use("bau","hh",r) - 1);
pc_co2_use("sum",r,psol_)$l_co2_use("bau","sum",r)
    = 100 * (l_co2_use(psol_,"sum",r) / l_co2_use("bau","sum",r) - 1);

pc_co2_so(en,r,psol_)$l_co2_so("bau",en,r)
    = 100 * (l_co2_so(psol_,en,r) / l_co2_so("bau",en,r) - 1);
pc_co2_so("sum",r,psol_)$l_co2_so("bau","sum",r)
    = 100 * (l_co2_so(psol_,"sum",r) / l_co2_so("bau","sum",r) - 1);

pc_co2_t(r,psol_)$l_co2_t(r,"bau")
    = 100 * (l_co2_t(r,psol_) / l_co2_t(r,"bau") - 1);
pc_co2_t("sum",psol_)$l_co2_t("sum","bau")
    = 100 * (l_co2_t("sum",psol_) / l_co2_t("sum","bau") - 1);
pc_co2_t("subj",psol_)$l_co2_t("subj","bau")
    = 100 * (l_co2_t("subj",psol_) / l_co2_t("subj","bau") - 1);
pc_co2_t("nsubj",psol_)$l_co2_t("nsubj","bau")
    = 100 * (l_co2_t("nsubj",psol_) / l_co2_t("nsubj","bau") - 1);

pc_tot(r,psol_) = 100 * (l_tot(psol_,r) / l_tot("bau",r) - 1);

r_summary("CO2 reduction target (MtCO2)",r,psol_) = - co2bench(r) * rd_rate(r) / 100 + eps;
r_summary("Permit price ($/tCO2)",r,psol_) = l_pco2_(psol_,r) + eps;
r_summary("Ch. in CO2 (MtCO2)",r,psol_) = d_co2(r,"MtCO2",psol_) + eps;
r_summary("%Ch. in CO2 (%)",r,psol_) = pc_co2_t(r,psol_) + eps;
r_summary("net permit imp (MtCO2)",r,psol_) = l_ptrade(psol_,r) + eps;
r_summary("v of net permit imp (bil$)",r,psol_) = l_vptrade(psol_,r) + eps;
r_summary("Value of permit (bil$)",r,psol_) = l_vperm(psol_,r) + eps;
r_summary("Welfare (%)",r,psol_) = pc_u(r,psol_) + eps;
r_summary("Real GDP (%)",r,psol_) = pc_gdp(r,psol_) + eps;
r_summary("Consumption (%)",r,psol_) = pc_c(r,psol_) + eps;
r_summary("Investment (%)",r,psol_) = pc_inv(r,psol_) + eps;
r_summary("Gov exp. (%)",r,psol_) = pc_gov(r,psol_) + eps;
r_summary("Export (%)",r,psol_) = pc_exp(r,psol_) + eps;
r_summary("Import (%)",r,psol_) = pc_imp(r,psol_) + eps;
r_summary("TOT (%)",r,psol_) = pc_tot(r,psol_) + eps;
r_summary("leakage rate (%)",r,psol_) = l_leak_rate("Leakage_rate (%)","%",psol_) + eps;
r_summary("Total gross production (%)",r,psol_) = pc_y("sum",r,psol_) + eps;
r_summary("Labor supply (%)",r,psol_) = pc_ls(r,psol_) + eps;
r_summary("Leisure (%)",r,psol_) = pc_lsrd(r,psol_) + eps;
r_summary("Wage rate (%)",r,psol_) = pc_pl(r,psol_) + eps;
r_summary("Wage rate for HH (%)",r,psol_) = pc_plh(r,psol_) + eps;
r_summary("Labor tax rate (%)",r,psol_) = l_labtax(psol_,r) + eps;
r_summary("Change in labor tax (point)",r,psol_) = 100 * (l_labtax(psol_,r) - l_labtax("bau",r)) + eps;
r_summary("Labor income (%)",r,psol_) = pc_labinc(r,psol_) + eps;
r_summary("HH income (%)",r,psol_) = pc_inc(r,psol_) + eps;
r_summary("Value of net permit rev",r,psol_) = l_vpermn(psol_,r) + eps;
r_summary("Rental price (%)",r,psol_) = pc_rk(r,psol_) + eps;

r_summary("Ch. in CO2 (MtCO2)","World",psol_) = d_co2("sum","MtCO2",psol_) + eps;
r_summary("%Ch. in CO2 (%)","World",psol_) = pc_co2_t("sum",psol_) + eps;
r_summary("net permit imp (MtCO2)","World",psol_) = l_ptrade(psol_,"sum") + eps;
r_summary("Welfare (%)","World",psol_) = pc_u("sum",psol_) + eps;
r_summary("Real GDP (%)","World",psol_) = pc_gdp("sum",psol_) + eps;

r_summary("Unemployment rate (%)",r,psol_) = l_ur(psol_,r) + eps;

display d_co2_leak_sec, d_co2_leak_reg, d_co2_leak, l_leak_rate_reg,
        l_leak_rate, d_y, d_co2_use, d_co2, l_vlabt, l_vlumpt, l_votht, l_vtax,
        l_vgov, d_nexp_c, pc_inv, pc_gov, pc_exp, pc_imp, pc_exp_c, pc_imp_c,
        pc_nexp_c, pc_y, pc_ld, pc_kd, pc_pl, pc_rk, pc_pya, pc_pym, pc_pene_ex,
        pc_labinc, pc_va_s, pc_co2, pc_co2_so, pc_co2_use, pc_co2_t;

abort$(smax(psol, chk_objval(psol) > 1e-4)) "Too large objval!!!", chk_objval;

display pc_u, pc_lsrd, pc_ls, pc_c, pc_gdp, pc_tot, l_ptrade, l_trb, l_trb_;
display r_summary;

*       ----------------------------------------------------------------------
*       Export results to an excel file:
display "@ Export results to an excel file:";

Execute_Unload '%gdx_f%', r_summary, l_leak_rate_reg,
    d_co2_leak_sec, d_co2_leak_reg, d_nexp_c, pc_ld, pc_pene_ex, pc_y, pc_co2_use, pc_u,
    pc_gdp, pc_va_s, d_y, d_co2_use, pc_exp_c, pc_imp_c, l_trb, l_trb_;

$label end

*       Delete temporary files.
execute 'del titlemaker_%s_name%.cmd';
execute 'del %s_name%_bau.txt';
execute 'del %s_name%_cmd.txt';
execute 'del %s_name%_cmd2.txt';
execute 'del iet_lab.gen';

$show

* --------------------
* local variables:
* mode: gams
* fill-column: 90
* end:
